'use strict';
const _ = require('lodash');

module.exports = (data) => {
  const { name, name_zh, columns } = data;
  let columnStr = `const ${name} = { \n`;
  let indexStr = '';
  let has_ObjectId = false;
  let has_Secret = false;
  const configArr = [];
  const updateArr = [];
  let configSearchStr = '';
  for (let i = 0; i < columns.length; i++) {
    const { title, type, required = false, remark, index, def, zh, ref, getProp } = columns[i];
    let refStr;
    if (ref) {
      refStr = ` ref: '${_.capitalize(ref)}'`;
      if (getProp) {
        const propList = getProp.split(';');
        let propStr = '[';
        for (const p of propList) {
          if (p !== _.head(propList)) propStr = `${propStr}, `;
          propStr = `${propStr}'${p}'`;
        }
        propStr += ']';
        refStr += `, getProp: ${propStr}`;
      }
    }
    const str = `  ${title}: { type: ${type || 'String'}, required: ${required} ${type === 'Secret' ? ', select: false' : ''} ${def ? `, default: '${def}'` : ''}${
      zh ? `, zh: '${zh}'` : ''
    }${refStr ? `,${refStr}` : ''} }, // ${remark || ''} \n`;
    columnStr += str;
    configArr.push(`'${required ? '!' : ''}${title}'`);
    updateArr.push(`'${title}'`);
    if (index) {
      indexStr += `schema.index({ '${title}': 1 });\n`;
      configSearchStr += `'${title}': '${title}' ,\n`;
    }
    if (!has_ObjectId && type === 'ObjectId') has_ObjectId = true;
    else if (!has_Secret && type === 'Secret') has_Secret = true;
  }
  columnStr += '}';
  const nameList = name.split('_');
  const modelName = nameList.map((i) => _.capitalize(i)).join('_');
  return `'use strict';
const Schema = require('mongoose').Schema;
const metaPlugin = require('naf-framework-mongoose-free/lib/model/meta-plugin');
${has_ObjectId ? "const { ObjectId } = require('mongoose').Types;" : ''}
${has_Secret ? "const { Secret } = require('naf-framework-mongoose-free/lib/model/schema');" : ''}
// ${name_zh || ''}
${columnStr}
const schema = new Schema(${name}, { toJSON: { virtuals: true } });
schema.index({ id: 1 });
schema.index({ 'meta.createdAt': 1 });
${indexStr}
schema.plugin(metaPlugin);
module.exports = app => {
  const { mongoose } = app;
  return mongoose.model('${modelName}', schema, '${name}');
};
  
module.exports = {
  create: {
    requestBody: [${configArr.join(',')}],
  },
  destroy: {
    params: ["!id"],
    service: "delete",
  },
  update: {
    params: ["!id"],
    requestBody: [${updateArr.join(',')}],
  },
  show: {
    parameters: {
      params: ["!id"],
    },
    service: "fetch",
  },
  index: {
    parameters: {
      query: {
        "meta.createdAt@start": "meta.createdAt@start",
        "meta.createdAt@end": "meta.createdAt@end",
        ${configSearchStr}
      },
      // options: {
      //   "meta.state": 0 // 默认条件
      // },
    },
    service: "query",
    options: {
      query: ["skip", "limit"],
      sort: ["meta.createdAt"],
      desc: true,
      count: true,
    },
  },
};
`;
};
